@@ -5,7 +5,7 @@ import numpy as np
 import torch
 import torch.nn as nn
 
-DETECTED_OUTPUT_TYPES = Union[Sequence[Any], Dict[Any, torch.Tensor], torch.Tensor]
+DETECTED_INPUT_OUTPUT_TYPES = Union[Sequence[Any], Dict[Any, torch.Tensor], torch.Tensor]
 
 
 class LayerInfo:
@@ -31,44 +31,59 @@ class LayerInfo:
         # Statistics
         self.trainable = True
         self.is_recursive = False
-        self.output_size = []  # type: List[Union[int, Sequence[Any], torch.Size]]
+        self.input_size = []  # type: List[int]
+        self.output_size = []  # type: List[int]
         self.kernel_size = []  # type: List[int]
         self.num_params = 0
         self.macs = 0
+        self.calculate_num_params()
 
     def __repr__(self) -> str:
         if self.depth_index is None:
             return "{}: {}".format(self.class_name, self.depth)
         return "{}: {}-{}".format(self.class_name, self.depth, self.depth_index)
 
-    def calculate_output_size(self, outputs: DETECTED_OUTPUT_TYPES, batch_dim: int) -> None:
-        """ Set output_size using the model's outputs. """
-        if isinstance(outputs, (list, tuple)):
-            try:
-                self.output_size = list(outputs[0].size())
-                self.output_size[batch_dim] = -1
-            except AttributeError:
-                # pack_padded_seq and pad_packed_seq store feature into data attribute
-                size = list(outputs[0].data.size())
-                self.output_size = size[:batch_dim] + [-1] + size[batch_dim + 1 :]
+    @staticmethod
+    def calculate_size(inputs: DETECTED_INPUT_OUTPUT_TYPES, batch_dim: Optional[int]) -> List[int]:
+        """ Set input_size or output_size using the model's inputs. """
+        if (
+            isinstance(inputs, (list, tuple))
+            and hasattr(inputs[0], "size")
+            and callable(inputs[0].size)
+        ):
+            size = list(inputs[0].size())
+            if batch_dim is not None:
+                size[batch_dim] = -1
 
-        elif isinstance(outputs, dict):
-            for _, output in outputs.items():
+        # pack_padded_seq and pad_packed_seq store feature into data attribute
+        elif isinstance(inputs, (list, tuple)) and hasattr(inputs[0], "data"):
+            size = list(inputs[0].data.size())
+            if batch_dim is not None:
+                size = size[:batch_dim] + [-1] + size[batch_dim + 1 :]
+
+        elif isinstance(inputs, dict):
+            # TODO avoid overwriting the previous size every time?
+            for _, output in inputs.items():
                 size = list(output.size())
-                size_with_batch = size[:batch_dim] + [-1] + size[batch_dim + 1 :]
-                self.output_size.append(size_with_batch)
+                if batch_dim is not None:
+                    size = [size[:batch_dim] + [-1] + size[batch_dim + 1 :]]
 
-        elif isinstance(outputs, torch.Tensor):
-            self.output_size = list(outputs.size())
-            self.output_size[batch_dim] = -1
+        elif isinstance(inputs, torch.Tensor):
+            size = list(inputs.size())
+            if batch_dim is not None:
+                size[batch_dim] = -1
 
         else:
             raise TypeError(
-                "Model contains a layer with an unsupported output type: {}".format(outputs)
+                "Model contains a layer with an unsupported input or output type: {}".format(inputs)
             )
 
+        return size
+
     def calculate_num_params(self) -> None:
-        """ Set num_params using the module's parameters.  """
+        """
+        Set num_params, trainable, inner_layers, and kernel_size using the module's parameters.
+        """
         for name, param in self.module.named_parameters():
             self.num_params += param.nelement()
             self.trainable &= param.requires_grad
@@ -80,20 +95,31 @@ class LayerInfo:
                     ksize[0], ksize[1] = ksize[1], ksize[0]
                 self.kernel_size = ksize
 
+            # RNN modules have inner weights such as weight_ih_l0
+            elif "weight" in name:
+                self.inner_layers[name] = list(param.size())
+
+    def calculate_macs(self) -> None:
+        """
+        Set MACs using the module's parameters and layer's output size, which is
+        used for computing number of operations for Conv layers.
+        """
+        for name, param in self.module.named_parameters():
+            if name == "weight":
                 # ignore N, C when calculate Mult-Adds in ConvNd
                 if "Conv" in self.class_name:
                     self.macs += int(param.nelement() * np.prod(self.output_size[2:]))
                 else:
                     self.macs += param.nelement()
-
             # RNN modules have inner weights such as weight_ih_l0
             elif "weight" in name:
-                self.inner_layers[name] = list(param.size())
                 self.macs += param.nelement()
 
-    def check_recursive(self, summary_list: "List[LayerInfo]") -> None:
-        """ if the current module is already-used, mark as (recursive).
-        Must check before adding line to the summary. """
+    def check_recursive(self, summary_list: List["LayerInfo"]) -> None:
+        """
+        If the current module is already-used, mark as (recursive).
+        Must check before adding line to the summary.
+        """
         if list(self.module.named_parameters()):
             for other_layer in summary_list:
                 if self.layer_id == other_layer.layer_id:
@@ -107,11 +133,10 @@ class LayerInfo:
 
     def num_params_to_str(self, reached_max_depth: bool = False) -> str:
         """ Convert num_params to string. """
-        assert self.num_params >= 0
         if self.is_recursive:
             return "(recursive)"
         if self.num_params > 0:
-            param_count_str = "{:,}".format((self.num_params))
+            param_count_str = "{:,}".format(self.num_params)
             if reached_max_depth or not any(self.module.children()):
                 if not self.trainable:
                     return "({})".format(param_count_str)
