@@ -1,5 +1,5 @@
 """ model_statistics.py """
-from typing import Any, Dict, List, Optional, Sequence, Union
+from typing import Any, Dict, List, Sequence, Union
 
 import numpy as np
 import torch
@@ -9,6 +9,7 @@ from .layer_info import LayerInfo
 
 HEADER_TITLES = {
     "kernel_size": "Kernel Shape",
+    "input_size": "Input Shape",
     "output_size": "Output Shape",
     "num_params": "Param #",
     "mult_adds": "Mult-Adds",
@@ -27,7 +28,7 @@ class ModelStatistics:
     ):
         self.summary_list = summary_list
         self.input_size = input_size
-        self.total_input = sum([abs(np.prod(sz)) for sz in input_size])
+        self.total_input = sum([abs(np.prod(sz)) for sz in input_size]) if input_size else 0
         self.formatting = formatting
         self.total_params, self.trainable_params = 0, 0
         self.total_output, self.total_mult_adds = 0, 0
@@ -48,13 +49,11 @@ class ModelStatistics:
     @staticmethod
     def to_bytes(num: int) -> float:
         """ Converts a number (assume floats, 4 bytes each) to megabytes. """
-        assert num >= 0
         return num * 4 / (1024 ** 2)
 
     @staticmethod
     def to_readable(num: int) -> float:
         """ Converts a number to millions or billions. """
-        assert num >= 0
         if num >= 1e9:
             return num / 1e9
         return num / 1e6
@@ -63,44 +62,38 @@ class ModelStatistics:
         """ Print results of the summary. """
         header_row = self.formatting.format_row("Layer (type:depth-idx)", HEADER_TITLES)
         layer_rows = self.layers_to_str()
-
-        total_size = self.total_input + self.total_output + self.total_params
-        width = self.formatting.get_total_width()
+        divider = "=" * self.formatting.get_total_width()
         summary_str = (
-            "{}\n"
-            "{}"
-            "{}\n"
-            "{}"
-            "{}\n"
-            "Total params: {:,}\n"
-            "Trainable params: {:,}\n"
-            "Non-trainable params: {:,}\n"
-            "Total mult-adds ({}): "
-            "{:0.2f}\n"
-            "{}\n"
-            "Input size (MB): {:0.2f}\n"
-            "Forward/backward pass size (MB): {:0.2f}\n"
-            "Params size (MB): {:0.2f}\n"
-            "Estimated Total Size (MB): {:0.2f}\n"
-            "{}".format(
-                "-" * width,
+            "{0}\n{1}{0}\n{2}{0}"
+            "\nTotal params: {3:,}\n"
+            "Trainable params: {4:,}\n"
+            "Non-trainable params: {5:,}\n".format(
+                divider,
                 header_row,
-                "=" * width,
                 layer_rows,
-                "=" * width,
                 self.total_params,
                 self.trainable_params,
                 self.total_params - self.trainable_params,
-                "G" if self.total_mult_adds >= 1e9 else "M",
-                self.to_readable(self.total_mult_adds),
-                "-" * width,
-                self.to_bytes(self.total_input),
-                self.to_bytes(self.total_output),
-                self.to_bytes(self.total_params),
-                self.to_bytes(total_size),
-                "-" * width,
             )
         )
+        if self.input_size:
+            summary_str += (
+                "Total mult-adds ({}): {:0.2f}\n"
+                "{}\n"
+                "Input size (MB): {:0.2f}\n"
+                "Forward/backward pass size (MB): {:0.2f}\n"
+                "Params size (MB): {:0.2f}\n"
+                "Estimated Total Size (MB): {:0.2f}\n".format(
+                    "G" if self.total_mult_adds >= 1e9 else "M",
+                    self.to_readable(self.total_mult_adds),
+                    divider,
+                    self.to_bytes(self.total_input),
+                    self.to_bytes(self.total_output),
+                    self.to_bytes(self.total_params),
+                    self.to_bytes(self.total_input + self.total_output + self.total_params),
+                )
+            )
+        summary_str += divider
         return summary_str
 
     def layer_info_to_row(self, layer_info: LayerInfo, reached_max_depth: bool = False) -> str:
@@ -111,6 +104,7 @@ class ModelStatistics:
 
         row_values = {
             "kernel_size": str(layer_info.kernel_size) if layer_info.kernel_size else "--",
+            "input_size": str(layer_info.input_size),
             "output_size": str(layer_info.output_size),
             "num_params": layer_info.num_params_to_str(reached_max_depth),
             "mult_adds": layer_info.macs_to_str(reached_max_depth),
@@ -146,10 +140,10 @@ class ModelStatistics:
 
             # create full hierarchy of current layer
             hierarchy = {}
-            parent_info = layer_info.parent_info
-            while parent_info is not None and parent_info.depth > 0:
-                hierarchy[parent_info.depth] = parent_info
-                parent_info = parent_info.parent_info
+            parent = layer_info.parent_info
+            while parent is not None and parent.depth > 0:
+                hierarchy[parent.depth] = parent
+                parent = parent.parent_info
 
             # show hierarchy if it is not there already
             for d in range(1, layer_info.depth):
